package org.kantega.jexmec;

/*
 * Copyright 2011 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.kantega.jexmec.events.PluginClassLoaderEvent;
import org.kantega.jexmec.events.PluginLoadingExceptionEvent;
import org.kantega.jexmec.events.PluginManagerEvent;
import org.kantega.jexmec.events.PluginRegistrationEvent;

import java.util.List;

/**
 * Listener class for PluginManager events.
 */
public abstract class PluginManagerListener<P> {

    /**
     * Fired when the plugin manager is about to be started
     * @param event the plugin manager event
     */
    public void beforePluginManagerStarted(PluginManagerEvent<P> event) {}

    /**
     * Fired after the plugin manager has started.
     * @param event the plugin manager event
     */
    public void afterPluginManagerStarted(PluginManagerEvent<P> event) {}

    /**
     * Fired when the plugin manager is about to be stopped
     * @param event the plugin manager event
     */
    public void beforePluginManagerStopped(PluginManagerEvent<P> event) {}

    /**
     * Fired when the plugin manager has been stopped
     * @param event the plugin manager event
     */
    public void afterPluginManagerStopped(PluginManagerEvent<P> event) {}

    /**
     * Fired when a plugin class loader provider requests a class loader to be added
     *
     * @param event the plugin class loader event
     */
    public void beforeClassLoaderAdded(PluginClassLoaderEvent<P> event) {}

    /**
     * Fired when the plugin manager has finished adding a class loader
     *
     * @param event the plugin class loader event
     */
    public void afterClassLoaderAdded(PluginClassLoaderEvent<P> event) {}

    /**
     * Fired when a plugin class loader provider requests a class loader to be removed
     *
     * @param event the plugin class loader event
     */
    public void beforeClassLoaderRemoved(PluginClassLoaderEvent<P> event) {}

    /**
     * Fired when the plugin manager has finished adding a class loader
     *
     * @param event the plugin class loader event
     */
    public void afterClassLoaderRemoved(PluginClassLoaderEvent<P> event) {}


    /**
     * Fired just before the plugins are activated (made available in the registry)
     * In this state, plugins are not yet visible to {@link PluginManager#getPlugins()}
     * @param event the plugin event
     */
    public void beforeActivation(PluginRegistrationEvent<P> event) {}

    /**
     * Fired when when plugins have been activated (made available in the registry). At this
     * point the plugins are visible to {@link PluginManager#getPlugins()}.
     * @param event the plugin event
     */
    public void afterActivation(PluginRegistrationEvent<P> event) {}


    /**
     * Fired when a collection of plugins is about to be passivated. At this point they have not been removed from the plugin
     * registry, and are visible to {@link PluginManager#getPlugins()}
     * @param event the plugin event
     */
    public void beforePassivation(PluginRegistrationEvent<P> event) {}

    /**
     * Fired when a collection of plugins have been passivated. That is they have been removed from the
     * plugin registry, but are not yet unloaded by their plugin loaders.
     * @param event the plugin event
     */
    public void afterPassivation(PluginRegistrationEvent<P> event) {}


    /**
     * Fired when the list of plugins has changed because plugins were added or removed.
     * @param plugins the updated list of plugins
     */
    public void pluginsUpdated(List<P> plugins) {}

    /**
     * Fired when a PluginLoader has thrown an exception during plugin loading
     *
     * @param event the plugin loading exception event
     */
    public void pluginLoadingFailedWithException(PluginLoadingExceptionEvent<P> event) {}




}


