package org.kantega.jexmec;

/*
 * Copyright 2011 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Key that identifies a service by type and optionally a name.
 */
public class ServiceKey<T> {
    private final Class<T> serviceType;
    private final String serviceName;

    /**
     * Creates a ServiceKey for a service of the given type and name.
     * This constructor is private, external code should use one of the
     * by() methods to create ServiceKeys. This is done to avoid redundant
     * specification of type parameter and the type class, like in:
     * <code>new ServiceKey<MyClass>(MyClass.class)</code>
     * @param serviceType the service type
     * @param serviceName the service name
     */
    private ServiceKey(Class<T> serviceType, String serviceName) {
        this.serviceType = serviceType;
        this.serviceName = serviceName;
    }

    /**
     * Creates a key for a service of type <code>type</code>
     * @param type the service Class
     * @param <T> the service key for the given type
     * @return a service key
     */
    public static <T> ServiceKey<T> by(Class<T> type) {
        return new ServiceKey<T>(type, null);
    }

    /**
     * Creates a key for a service of type <code>type</code> and name <code>name</code>
     * @param type the service Class
     * @param <T> the service type
     * @param name the service name
     * @return the service key for the given type and name
     */
    public static <T> ServiceKey<T> by(Class<T> type, String name) {
        return new ServiceKey<T>(type, name);
    }

    /**
     * Returns the type of the service.
     * @return the type of the service
     */
    public Class<T> getServiceType() {
        return serviceType;
    }

    /**
     * Returns the name of the service or <code>null</code> if the service is not a named service.
     * @return the name of the service or <code>null</code> if the service is not a named service.
     */
    public String getServiceName() {
        return serviceName;
    }

    @Override
    public int hashCode() {
        if(serviceName == null) {
            return serviceType.hashCode();
        } else {
            return serviceType.hashCode() ^ serviceName.hashCode();
        }
    }

    @Override
    public boolean equals(Object o) {
        if(o.getClass() != ServiceKey.class) {
            return false;
        }
        ServiceKey key = ServiceKey.class.cast(o);

        if(serviceType != key.getServiceType()) {
            return false;
        }
        if(serviceName == null && key.getServiceName() != null) {
            return false;
        }
        if(serviceName != null && key.getServiceName() == null) {
            return false;
        }

        if(serviceName != null && !serviceName.equals(key.getServiceName())) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        if(getServiceName() == null) {
            return "[type=" +getServiceType().getName() +"]";
        } else {
            return "[type=" +getServiceType().getName() +", name='" +getServiceName() + "']";
        }
    }
}
